//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/AxesItems.cpp
//! @brief     Implements various axis items
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Item/AxesItems.h"
#include "Base/Axis/FixedBinAxis.h"
#include "GUI/Model/XML/Serializer.h"

const int max_detector_pixels = 65536;

BasicAxisItem::BasicAxisItem()
    : BasicAxisItem(M_TYPE)
{
}

int BasicAxisItem::binCount() const
{
    return getItemValue(P_NBINS).toInt();
}

void BasicAxisItem::setBinCount(size_t value)
{
    setItemValue(P_NBINS, (int)value);
}

SessionItem* BasicAxisItem::binsItem() const
{
    return getItem(P_NBINS);
}

DoubleDescriptor BasicAxisItem::lowerBound(const QString& unit) const
{
    DoubleDescriptor d(getItem(P_MIN_DEG), unit);
    d.label = "Min";
    return d;
}

void BasicAxisItem::setLowerBound(double value)
{
    setItemValue(P_MIN_DEG, value);
}

SessionItem* BasicAxisItem::lowerBoundItem() const
{
    return getItem(P_MIN_DEG);
}

DoubleDescriptor BasicAxisItem::upperBound(const QString& unit) const
{
    DoubleDescriptor d(getItem(P_MAX_DEG), unit);
    d.label = "Max";
    return d;
}

void BasicAxisItem::setUpperBound(double value)
{
    setItemValue(P_MAX_DEG, value);
}

SessionItem* BasicAxisItem::upperBoundItem() const
{
    return getItem(P_MAX_DEG);
}

bool BasicAxisItem::isBoundsPropertiesName(const QString& name)
{
    return name == P_MIN_DEG || name == P_MAX_DEG;
}

QString BasicAxisItem::title() const
{
    return getItemValue(P_TITLE).toString();
}

void BasicAxisItem::setTitle(const QString& title)
{
    setItemValue(P_TITLE, title);
}

SessionItem* BasicAxisItem::titleItem() const
{
    return getItem(P_TITLE);
}

bool BasicAxisItem::isTitlePropertyName(const QString& name)
{
    return name == P_TITLE;
}

bool BasicAxisItem::isTitleVisible() const
{
    return getItemValue(P_TITLE_IS_VISIBLE).toBool();
}

bool BasicAxisItem::isTitleVisiblePropertyName(const QString& name)
{
    return name == P_TITLE_IS_VISIBLE;
}

std::unique_ptr<IAxis> BasicAxisItem::createAxis(double scale) const
{
    return std::make_unique<FixedBinAxis>(title().toStdString(), binCount(), lowerBound() * scale,
                                          upperBound() * scale);
}

SessionItem* BasicAxisItem::visibilityItem() const
{
    return getItem(P_IS_VISIBLE);
}

bool BasicAxisItem::visibilityValue() const
{
    return visibilityItem()->value().toBool();
}

void BasicAxisItem::setVisibilityValue(bool b)
{
    setItemValue(P_IS_VISIBLE, b);
}

bool BasicAxisItem::isVisibilityPropertyName(const QString& name)
{
    return name == P_IS_VISIBLE;
}

BasicAxisItem::~BasicAxisItem() = default;

BasicAxisItem::BasicAxisItem(const QString& type)
    : SessionItem(type)
{
    addProperty(P_IS_VISIBLE, true);
    addProperty(P_NBINS, 100)->setLimits(RealLimits::limited(1, max_detector_pixels));
    addProperty(P_MIN_DEG, 0.0)->setDecimals(3).setLimits(RealLimits::limitless());
    addProperty(P_MAX_DEG, -1.0)->setDecimals(3).setLimits(RealLimits::limitless());
    addProperty(P_TITLE, QString());
    addProperty(P_TITLE_IS_VISIBLE, true);
}

void BasicAxisItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw<bool>("visible", getItem(P_IS_VISIBLE));
    s.rw<int>("nbins", getItem(P_NBINS));
    s.rw<double>("min", getItem(P_MIN_DEG));
    s.rw<double>("max", getItem(P_MAX_DEG));
    s.rw<QString>("title", getItem(P_TITLE));
    s.rw<bool>("titleVisible", getItem(P_TITLE_IS_VISIBLE));
}

// ---------------------------------------------------------------------------------------------- //

AmplitudeAxisItem::AmplitudeAxisItem()
    : BasicAxisItem(M_TYPE)
{
    addProperty(P_LOCK_MIN_MAX, false);
    addProperty(P_IS_LOGSCALE, true);
    visibilityItem();
}

void AmplitudeAxisItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rwBaseClass<BasicAxisItem>("base", this);
    s.rw<bool>("lockMinMax", getItem(P_LOCK_MIN_MAX));
    s.rw<bool>("log", getItem(P_IS_LOGSCALE));
}

bool AmplitudeAxisItem::isLogScale() const
{
    return getItemValue(P_IS_LOGSCALE).toBool();
}

void AmplitudeAxisItem::setLogScale(bool value)
{
    setItemValue(P_IS_LOGSCALE, value);
}

SessionItem* AmplitudeAxisItem::logScaleItem() const
{
    return getItem(P_IS_LOGSCALE);
}

bool AmplitudeAxisItem::isLogScalePropertyName(const QString& name)
{
    return name == P_IS_LOGSCALE;
}

bool AmplitudeAxisItem::isLocked() const
{
    return getItemValue(P_LOCK_MIN_MAX).toBool();
}

void AmplitudeAxisItem::setLocked(bool locked)
{
    setItemValue(P_LOCK_MIN_MAX, locked);
}
